﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using System.Xml.Linq;
using AppReviews.Web;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Tasks;

namespace AppReviews
{
    public partial class AddAppPage : PhoneApplicationPage
    {
        public AddAppPage()
        {
            InitializeComponent();
        }

        WebWorkItem _currentRequest = null;
        bool _isInPage = false;
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            CancelCurrentRequestIfNeeded();
            WebApi.CancelAll();
            _isInPage = true;
            textAppName.Text = Globals.SearchTerm;
            if (!string.IsNullOrEmpty(textAppName.Text))
            {
                SearchForApp();
            }
        }

        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);
            _isInPage = false;
            Globals.SearchTerm = textAppName.Text;
            CancelCurrentRequestIfNeeded();
        }

        private void CancelCurrentRequestIfNeeded()
        {
            if (_currentRequest != null)
            {
                WebApi.Cancel(_currentRequest);
                _currentRequest = null;
            }
        }

        private void textAppName_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                listApps.Focus();
                SearchForApp();
            }
        }

        private void SearchForApp()
        {
            string searchText = textAppName.Text;
            if (string.IsNullOrEmpty(searchText))
            {
                MessageBox.Show("Please enter search text");
                return;
            }

            CancelCurrentRequestIfNeeded();
            errorControl.ErrorMessage = null;
            SetLoadingState(true);
            _currentRequest = WebApi.QueueAppSearch(searchText, null, null, SearchCompleted);
        }

        private void SetLoadingState(bool isLoading)
        {
            if (isLoading)
            {
                progressLoad.Visibility = Visibility.Visible;
                progressLoad.IsLoading = true;
            }
            else
            {
                progressLoad.Visibility = Visibility.Collapsed;
                progressLoad.IsLoading = false;
            }
        }

        private void SearchCompleted(WebWorkItem item)
        {
            if (!_isInPage) // check if the user navigated away during search
            {
                return;
            }

            Dispatcher.BeginInvoke(delegate
            {
                SetLoadingState(false);

                if (item.Error != null)
                {
                    errorControl.ErrorMessage = "Connection error. Try loading data again later. Error message: " + item.Error.Message;
                    return;
                }

                if (item.Result == null) // cancelled
                {
                    return;
                }

                // process result
                try
                {
                    ParseSearchResults(item.Result);
                }
                catch (Exception e)
                {
                    WebApi.CancelAll();
                    Globals.ErrorMessage = "Incorrect marketplace data. Try loading data again later. Error message: " + e.Message;
                }
            });
        }

        const string atom = "{http://www.w3.org/2005/Atom}";

        static XName entryName = atom + "entry";

        private void ParseSearchResults(string xml)
        {
            XDocument document = XDocument.Parse(xml);
           
            List<ApplicationData> data = new List<ApplicationData>();
            foreach (XElement entry in document.Root.Elements(entryName))
            {
                ApplicationData item = new ApplicationData(false);
                item.LoadAppData(entry);
                data.Add(item);
            }

            listApps.ItemsSource = data;
            if (data.Count == 0)
            {
                errorControl.ErrorMessage = "No results found, try another search string, such as application name or keyword";
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (listApps.SelectedItem == null) return;
            ApplicationData item = (ApplicationData)listApps.SelectedItem;
            item.InitCollections();
            Globals.Data.Apps.Add(item);
            NavigationService.Navigate(Globals.MainPageUri);
        }

        private void listApps_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            ((ApplicationBarIconButton)ApplicationBar.Buttons[0]).IsEnabled = listApps.SelectedIndex >= 0;
            ((ApplicationBarIconButton)ApplicationBar.Buttons[1]).IsEnabled = listApps.SelectedIndex >= 0;
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            listApps.Focus();
            SearchForApp();
        }

        private void textAppName_GotFocus(object sender, RoutedEventArgs e)
        {
            textAppName.SelectAll();
        }
        
        MarketplaceDetailTask _marketTask = null;
        private void btnOpen_Click(object sender, EventArgs e)
        {
            if (listApps.SelectedIndex < 0)
            {
                return;
            }

            ApplicationData app = (ApplicationData)listApps.SelectedItem;
            if (_marketTask == null) _marketTask = new MarketplaceDetailTask();
            _marketTask.ContentType = MarketplaceContentType.Applications;
            _marketTask.ContentIdentifier = app.Id;
            _marketTask.Show();
        }
    }
}