﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using AppReviews.Web;
using System.Windows;
using System.Xml.Linq;
using System.Linq;
using System.Threading;
using System.Diagnostics;

namespace AppReviews
{
    public class ApplicationData : INotifyPropertyChanged
    {
        bool _appDataLoaded = false;
        public ApplicationData()
            : this(true)
        {
        }

        public ApplicationData(bool initCollections)
        {
            if (initCollections)
            {
                InitCollections();
            }
        }

        public void InitCollections()
        {
            Markets = new ObservableCollection<MarketplaceData>();
            Reviews = new ObservableCollection<ReviewData>();
        }

        private string _name;
        public string Name { get { return _name; } set { _name = value; NotifyPropertyChanged("Name"); } }

        private string _id;
        public string Id { get { return _id; } set { _id = value; NotifyPropertyChanged("Id"); } }

        private string _versionAndDeveloper;
        public string VersionAndDeveloper { get { return _versionAndDeveloper; } set { _versionAndDeveloper = value; NotifyPropertyChanged("VersionAndDeveloper"); } }

        private string _category;
        public string Category { get { return _category; } set { _category = value; NotifyPropertyChanged("Category"); } }

        private string _releasedDate;
        public string ReleasedDate { get { return _releasedDate; } set { _releasedDate = value; NotifyPropertyChanged("ReleasedDate"); } }

        private string _imageUrl;
        public string ImageUrl { get { return _imageUrl; } set { _imageUrl = value; NotifyPropertyChanged("ImageUrl"); } }

        private double _averageRating;
        public double AverageRating { get { return _averageRating; } set { _averageRating = value; NotifyPropertyChanged("AverageRating"); } }

        private int _ratingCount;
        public int RatingCount { get { return _ratingCount; } set { _ratingCount = value; NotifyPropertyChanged("RatingCount"); } }

        private double _averageRatingUpdate;
        public double AverageRatingUpdate { get { return _averageRatingUpdate; } set { _averageRatingUpdate = value; NotifyPropertyChanged("AverageRatingUpdate"); } }

        private int _ratingCountUpdate;
        public int RatingCountUpdate { get { return _ratingCountUpdate; } set { _ratingCountUpdate = value; NotifyPropertyChanged("RatingCountUpdate"); } }

        private string _ratingStringAll;
        public string RatingStringAll { get { return _ratingStringAll; } set { _ratingStringAll = value; NotifyPropertyChanged("RatingStringAll"); } }

        private string _ratingStringUpdate;
        public string RatingStringUpdate { get { return _ratingStringUpdate; } set { _ratingStringUpdate = value; NotifyPropertyChanged("RatingStringUpdate"); } }

        private string _priceString;
        public string PriceString { get { return _priceString; } set { _priceString = value; NotifyPropertyChanged("PriceString"); } }

        private Visibility _ratingVisibilityAll;
        public Visibility RatingVisibilityAll { get { return _ratingVisibilityAll; } set { _ratingVisibilityAll = value; NotifyPropertyChanged("RatingVisibilityAll"); } }

        private Visibility _ratingVisibilityUpdate;
        public Visibility RatingVisibilityUpdate { get { return _ratingVisibilityUpdate; } set { _ratingVisibilityUpdate = value; NotifyPropertyChanged("RatingVisibilityUpdate"); } }

        private bool _autoLoadReviews = false;
        private object _groupId = new object();

        public void LoadDataFirstPass(bool preloadNameFirst)
        {
            Markets.Clear();
            Reviews.Clear();
            RatingCount = 0;
            AverageRating = 0;
            RatingCountUpdate = 0;
            AverageRatingUpdate = 0;
            _autoLoadReviews = false;

            this.RatingStringAll = "loading";
            this.RatingVisibilityAll = Visibility.Collapsed;
            this.RatingStringUpdate = "loading";
            this.RatingVisibilityUpdate = Visibility.Collapsed;
            
            AllMarkets = new MarketplaceData(null, "All Markets", null);

            Markets.Add(AllMarkets);
            MarketplaceData usMarket = new MarketplaceData("English", "United States", "en-us");
            Markets.Add(usMarket);
            Markets.Add(new MarketplaceData("English", "Australia", "en-au"));
            Markets.Add(new MarketplaceData("English", "Canada", "en-ca"));
            Markets.Add(new MarketplaceData("English", "Great Britain", "en-gb"));
            Markets.Add(new MarketplaceData("English", "Hong Kong", "en-hk"));
            Markets.Add(new MarketplaceData("English", "India", "en-in"));
            Markets.Add(new MarketplaceData("English", "Ireland", "en-ie"));
            Markets.Add(new MarketplaceData("English", "New Zealand", "en-nz"));
            Markets.Add(new MarketplaceData("English", "Singapore", "en-sg"));
            Markets.Add(new MarketplaceData("English", "South Africa", "en-za"));
            Markets.Add(new MarketplaceData("German", "Germany", "de-de"));
            Markets.Add(new MarketplaceData("German", "Austria", "de-at"));
            Markets.Add(new MarketplaceData("German", "Switzerland", "de-ch"));
            Markets.Add(new MarketplaceData("French", "Belgium", "fr-be"));
            Markets.Add(new MarketplaceData("French", "Canada", "fr-ca"));
            Markets.Add(new MarketplaceData("French", "France", "fr-fr"));
            Markets.Add(new MarketplaceData("French", "Switzerland", "fr-ch"));
            Markets.Add(new MarketplaceData("Spanish", "Chile", "es-cl"));
            Markets.Add(new MarketplaceData("Spanish", "Colombia", "es-co"));
            Markets.Add(new MarketplaceData("Spanish", "Mexico", "es-mx"));
            Markets.Add(new MarketplaceData("Spanish", "Spain", "es-es"));
            Markets.Add(new MarketplaceData("Chinese", "Hong Kong", "zh-hk"));
            Markets.Add(new MarketplaceData("Chinese", "Taiwan", "zh-tw"));

            Markets.Add(new MarketplaceData("Czech", "Czech Republic", "cs-cz"));
            Markets.Add(new MarketplaceData("Danish", "Denmark", "da-dk"));
            Markets.Add(new MarketplaceData("Dutch", "Netherlands", "nl-nl"));
            Markets.Add(new MarketplaceData("Dutch", "Belgium", "nl-be"));
            Markets.Add(new MarketplaceData("Finnish", "Finland", "fi-fi"));
            Markets.Add(new MarketplaceData("Greek", "Greece", "el-gr"));
            Markets.Add(new MarketplaceData("Hungarian", "Hungary", "hu-hu"));
            Markets.Add(new MarketplaceData("Italian", "Italy", "it-it"));
            Markets.Add(new MarketplaceData("Japanese", "Japan", "ja-jp"));
            Markets.Add(new MarketplaceData("Korean", "South Korea", "ko-kr"));
            Markets.Add(new MarketplaceData("Norwegian", "Norway", "nb-no"));
            Markets.Add(new MarketplaceData("Polish", "Poland", "pl-pl"));
            Markets.Add(new MarketplaceData("Portuguese", "Brazil", "pt-br"));
            Markets.Add(new MarketplaceData("Portuguese", "Portugal", "pt-pt"));
            Markets.Add(new MarketplaceData("Russian", "Russia", "ru-ru"));
            Markets.Add(new MarketplaceData("Swedish", "Sweden", "sv-se"));

            if (preloadNameFirst)
            {
                WebApi.QueueGetAppData(usMarket.LocaleId, Id, usMarket, null, GotAppData);
            }
            else
            {
                WebApi.QueueGetAppData(usMarket.LocaleId, Id, usMarket, _groupId, GotAppData);
            }
        }

        public void LoadDataSecondPass(bool preloadNameFirst)
        {
            foreach (MarketplaceData marketplace in Markets)
            {
                if ((marketplace.LocaleId == null) || marketplace.LocaleId.Equals("en-us", StringComparison.InvariantCultureIgnoreCase)) continue;
                WebApi.QueueGetAppData(marketplace.LocaleId, Id, marketplace, _groupId, GotAppData);
            }
        }

        public void LoadReviews()
        {
            foreach (MarketplaceData market in Markets)
            {
                if (market.LocaleId == null) continue;

                if (market.CountsLoaded)
                {
                    if ((market.RatingCount > 0) && (!market.ReviewsLoaded) && (!market.IsLoadingReviews))
                    {
                        market.IsLoadingReviews = true;
                        WebApi.QueueGetAppReviews(market.LocaleId, Id, market, _groupId, GotAppReviews);
                    }
                }
                else
                {
                    _autoLoadReviews = true;
                }
            }
        }

        MarketplaceData AllMarkets = null;

        const string atom = "{http://www.w3.org/2005/Atom}";
        const string zune = "{http://schemas.zune.net/catalog/apps/2008/02}";

        static XName entryName = atom + "entry";
        static XName categoriesName = zune + "categories";
        static XName categoryName = zune + "category";
        static XName titleName = zune + "title";
        static XName publisherName = zune + "publisher";
        static XName nameName = zune + "name";
        static XName idName = zune + "id";
        static XName idNameAtom = atom + "id";
        static XName titleNameAtom = atom + "title";
        static XName imageName = zune + "image";
        static XName offsersName = zune + "offers";
        static XName releaseDateName = zune + "releaseDate";
        static XName versionName = zune + "version";
        static XName averageUserRatingName = zune + "averageUserRating";
        static XName userRatingCountName = zune + "userRatingCount";
        static XName averageUserRatingLastUpdateName = zune + "averageLastInstanceUserRating";
        static XName userRatingCountLastUpdateName = zune + "lastInstanceUserRatingCount";
        static XName updatedName = atom + "updated";
        static XName offerName = zune + "offer";
        static XName priceName = zune + "price";
        static XName displayPriceName = zune + "displayPrice";
        static XName licenseRightName = zune + "licenseRight";

        public void LoadAppData(XElement entry)
        {
            Category = (string)entry.Element(categoriesName).Element(categoryName).Element(titleName);
            XElement publisher = entry.Element(publisherName);
            if (publisher.HasElements)
            {
                VersionAndDeveloper = string.Format("{0}, {1}", (string)entry.Element(versionName), (string)publisher.Element(nameName));
            }
            else
            {
                VersionAndDeveloper = string.Format("{0}, {1}", (string)entry.Element(entryName).Element(versionName), (string)publisher);
            }
            Id = ((string)entry.Element(idNameAtom)).Substring(9);
            ImageUrl = string.Format("http://image.catalog.zune.net/v3.2/image/{0}?width=100&height=100", ((string)entry.Element(imageName).Element(idName)).Substring(9));
            Name = (string)entry.Element(titleNameAtom);

            double purchasePrice = 0;
            string displayPurchasePrice = "0";
            double trialPrice = -1;
            string displayTrialPrice = "0";
            foreach (XElement offer in entry.Element(offsersName).Elements(offerName))
            {
                string licenseRight = ((string)offer.Element(licenseRightName));

                if (licenseRight.Equals("Purchase", StringComparison.InvariantCultureIgnoreCase))
                {
                    purchasePrice = (double)offer.Element(priceName);
                    displayPurchasePrice = (string)offer.Element(displayPriceName);
                    continue;
                }

                if (licenseRight.Equals("Trial", StringComparison.InvariantCultureIgnoreCase))
                {
                    trialPrice = (double)offer.Element(priceName);
                    displayTrialPrice = (string)offer.Element(displayPriceName);
                    continue;
                }
            }

            if (purchasePrice == 0)
            {
                PriceString = "Free";
            }
            else
            {
                if (trialPrice == 0)
                {
                    PriceString = string.Format("Free to try, {0} to buy", displayPurchasePrice);
                }
                else
                {
                    PriceString = displayPurchasePrice;
                }
            }

            ReleasedDate = ((DateTime)entry.Element(releaseDateName)).ToShortDateString();
            AverageRating = ((double)entry.Element(averageUserRatingName)) / 2;
            RatingCount = (int)entry.Element(userRatingCountName);
            AverageRatingUpdate = ((double)entry.Element(averageUserRatingLastUpdateName)) / 2;
            RatingCountUpdate = (int)entry.Element(userRatingCountLastUpdateName);

            UpdateStringRatings();
        }

        private void UpdateStringRatings()
        {
            if (this.RatingCount == 0)
            {
                this.RatingStringAll = "not rated";
                this.RatingVisibilityAll = Visibility.Collapsed;
            }
            else
            {
                this.RatingStringAll = string.Format("{0}/{1:0.0}", this.RatingCount, this.AverageRating);
                this.RatingVisibilityAll = Visibility.Visible;
            }

            if (this.RatingCountUpdate == 0)
            {
                this.RatingStringUpdate = "not rated";
                this.RatingVisibilityUpdate = Visibility.Collapsed;
            }
            else
            {
                this.RatingStringUpdate = string.Format("{0}/{1:0.0}", this.RatingCountUpdate, this.AverageRatingUpdate);
                this.RatingVisibilityUpdate = Visibility.Visible;
            }
        }

        /// <summary>
        /// Move the queries for this application to the top of the queue
        /// </summary>
        public void MoveToTop()
        {
            WebApi.MoveToTop(_groupId);
        }

        public void GotAppData(WebWorkItem item)
        {
            Globals.Dispatcher.BeginInvoke(delegate
            {
                if (item.Error != null)
                {
                    Globals.ErrorMessage = "Connection error or app '" + Name + "' not found. Try loading data again later. Error message: " +
                        item.Error.Message;
                    return;
                }

                if (item.Result == null) // cancelled
                {
                    return;
                }

                MarketplaceData market = (MarketplaceData)item.UserData;

                //if (market.LocaleId.Equals("en-us", StringComparison.InvariantCultureIgnoreCase))
                //{
                //    foreach (MarketplaceData marketplace in Markets)
                //    {
                //        if ((marketplace.LocaleId == null) || marketplace.LocaleId.Equals("en-us", StringComparison.InvariantCultureIgnoreCase)) continue;
                //        WebApi.QueueGetAppData(marketplace.LocaleId, Id, marketplace, _groupId, GotAppData);
                //    }
                //}

                // process result
                try
                {
                    XDocument document = XDocument.Parse(item.Result);
                    if (!_appDataLoaded || market.LocaleId.Equals("en-us", StringComparison.InvariantCultureIgnoreCase))
                    {
                        LoadAppData(document.Root);
                        _appDataLoaded = true;
                    }
                    market.AverageRating = ((double)document.Root.Element(averageUserRatingName)) / 2;
                    market.RatingCount = (int)document.Root.Element(userRatingCountName);

                    market.AverageRatingSinceLastUpdate = ((double)document.Root.Element(averageUserRatingLastUpdateName)) / 2;
                    market.RatingCountSinceLastUpdate = (int)document.Root.Element(userRatingCountLastUpdateName);

                    bool shouldUpdateStringRatings = false;
                    if (market.RatingCount > 0)
                    {
                        if (AllMarkets != null)
                        {
                            // aggregate rating
                            double totalRating = AllMarkets.AverageRating * AllMarkets.RatingCount +
                                market.AverageRating * market.RatingCount;

                            AllMarkets.RatingCount += market.RatingCount;
                            Debug.WriteLine("Market: {0}, Count: {1}, TotalCount: {2}", market.MarketName, market.RatingCount, AllMarkets.RatingCount);
                            AllMarkets.AverageRating = totalRating / AllMarkets.RatingCount;

                            this.RatingCount = AllMarkets.RatingCount;
                            this.AverageRating = AllMarkets.AverageRating;
                            shouldUpdateStringRatings = true;
                        }
                        if (_autoLoadReviews && (!market.IsLoadingReviews))
                        {
                            market.IsLoadingReviews = true;
                            WebApi.QueueGetAppReviews(market.LocaleId, Id, market, _groupId, GotAppReviews);
                        }
                    }

                    if (market.RatingCountSinceLastUpdate > 0)
                    {
                        if (AllMarkets != null)
                        {
                            // aggregate rating
                            double totalRating = AllMarkets.AverageRatingSinceLastUpdate * AllMarkets.RatingCountSinceLastUpdate +
                                market.AverageRatingSinceLastUpdate * market.RatingCountSinceLastUpdate;

                            AllMarkets.RatingCountSinceLastUpdate += market.RatingCountSinceLastUpdate;
                            AllMarkets.AverageRatingSinceLastUpdate = totalRating / AllMarkets.RatingCountSinceLastUpdate;

                            this.RatingCountUpdate = AllMarkets.RatingCountSinceLastUpdate;
                            this.AverageRatingUpdate = AllMarkets.AverageRatingSinceLastUpdate;
                            shouldUpdateStringRatings = true;
                        }
                    }

                    if (shouldUpdateStringRatings)
                    {
                        UpdateStringRatings();
                    }
                    market.CountsLoaded = true;
                }
                catch (Exception e)
                {
                    WebApi.CancelAll();
                    Globals.ErrorMessage = "Incorrect marketplace data. Try loading data again later. Error message: " + e.Message;
                }
            });
        }

        public ObservableCollection<ReviewData> Reviews { get; private set; }

        public void GotAppReviews(WebWorkItem item)
        {
            MarketplaceData market = (MarketplaceData)item.UserData;
            market.IsLoadingReviews = false;

            if (item.Result == null) // cancelled
            {
                return;
            }

            market.ReviewsLoaded = true;

            Globals.Dispatcher.BeginInvoke(delegate
            {
                if (item.Error != null)
                {
                    WebApi.CancelAll();
                    Globals.ErrorMessage = "Connection error or reviews not found. Try loading data again later. Error message: " +
                        item.Error.Message;
                    return;
                }

                try
                {
                    XDocument document = XDocument.Parse(item.Result);
                    XNamespace atomNamespace = "http://www.w3.org/2005/Atom";
                    XNamespace zuneNamespace = "http://schemas.zune.net/catalog/apps/2008/02";

                    var entries = from entry in document.Root.Elements(atomNamespace + "entry")
                                  select new ReviewData()
                                  {
                                      ReviewText = (string)entry.Element(atomNamespace + "content"),
                                      Title = (string)entry.Element(atomNamespace + "title"),
                                      Date = (DateTime)entry.Element(atomNamespace + "updated"),
                                      Author = (string)entry.Element(atomNamespace + "author"),
                                      Rating = ((int)entry.Element(zuneNamespace + "userRating")) / 2,
                                      MarketName = market.MarketName,
                                  };
                    foreach (ReviewData entry in entries)
                    {
                        AddEntrySorted(entry);
                    }
                }
                catch (Exception e)
                {
                    WebApi.CancelAll();
                    Globals.ErrorMessage = "Incorrect review data. Try loading data again later. Error message: " + e.Message;
                }
            });
        }

        private void AddEntrySorted(ReviewData entry)
        {
            int s = 0;
            int e = Reviews.Count - 1;
            while (s <= e)
            {
                int m = (s + e) / 2;
                ReviewData middle = Reviews[m];
                if (middle.Date > entry.Date)
                {
                    s = m + 1;
                }
                else
                {
                    e = m - 1;
                }
            }

            Reviews.Insert(s, entry);
        }

        public ObservableCollection<MarketplaceData> Markets { get; private set; }

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
