﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;

namespace AppReviews
{
    public class MarketplaceData : INotifyPropertyChanged
    {
        public MarketplaceData()
        {
        }

        public string LocaleId;

        public bool IsLoadingReviews = false;

        public MarketplaceData(string language, string country, string localeId)
            : this()
        {
            LocaleId = localeId;
            MarketName = country;
            if (language != null)
            {
                _marketLanguageAndName = string.Format("{0} - {1}", language, country);
            }
            else
            {
                _marketLanguageAndName = country;
            }
        }

        public bool ReviewsLoaded = false;
        public bool CountsLoaded = false;

        private string _marketName;
        public string MarketName
        {
            get
            {
                return _marketName;
            }
            set
            {
                if (value != _marketName)
                {
                    _marketName = value;
                    NotifyPropertyChanged("MarketName");
                }
            }
        }

        private string _marketLanguageAndName;
        public string MarketLanguageAndName
        {
            get
            {
                return _marketLanguageAndName;
            }
            set
            {
                if (value != _marketLanguageAndName)
                {
                    _marketName = value;
                    NotifyPropertyChanged("MarketLanguageAndName");
                }
            }
        }

        private double _averageRating;
        public double AverageRating
        {
            get
            {
                return _averageRating;
            }
            set
            {
                if (value != _averageRating)
                {
                    _averageRating = value;
                    NotifyPropertyChanged("Rating");
                    NotifyPropertyChanged("RatingString");
                }
            }
        }

        private int _ratingCount;
        public int RatingCount
        {
            get
            {
                return _ratingCount;
            }
            set
            {
                if (value != _ratingCount)
                {
                    _ratingCount = value;
                    NotifyPropertyChanged("RatingString");
                    NotifyPropertyChanged("RatingVisibility");
                }
            }
        }

        private double _averageRatingSinceLastUpdate;
        public double AverageRatingSinceLastUpdate
        {
            get
            {
                return _averageRatingSinceLastUpdate;
            }
            set
            {
                if (value != _averageRatingSinceLastUpdate)
                {
                    _averageRatingSinceLastUpdate = value;
                    NotifyPropertyChanged("Rating");
                    NotifyPropertyChanged("RatingString");
                }
            }
        }

        private int _ratingCountSinceLastUpdate;
        public int RatingCountSinceLastUpdate
        {
            get
            {
                return _ratingCountSinceLastUpdate;
            }
            set
            {
                if (value != _ratingCountSinceLastUpdate)
                {
                    _ratingCountSinceLastUpdate = value;
                    NotifyPropertyChanged("RatingString");
                    NotifyPropertyChanged("RatingVisibility");
                }
            }
        }

        public double Rating
        {
            get
            {
                if (_showAllRatings)
                {
                    return _averageRating;
                }
                else
                {
                    return _averageRatingSinceLastUpdate;
                }
            }
        }

        private bool _showAllRatings = true;
        public bool ShowAllRatings
        {
            get
            {
                return _showAllRatings;
            }
            set
            {
                if (_showAllRatings == value) return;
                _showAllRatings = value;
                NotifyPropertyChanged("Rating");
                NotifyPropertyChanged("RatingString");
                NotifyPropertyChanged("RatingVisibility");
            }
        }

        public Visibility RatingVisibility
        {
            get
            {
                int count;
                if (_showAllRatings)
                {
                    count = _ratingCount;
                }
                else
                {
                    count = _ratingCountSinceLastUpdate;
                }

                if (count == 0)
                {
                    return Visibility.Collapsed;
                }

                return Visibility.Visible;
            }
        }

        public string RatingString
        {
            get
            {
                int count;
                double average;
                if (_showAllRatings)
                {
                    count = _ratingCount;
                    average = _averageRating;
                }
                else
                {
                    count = _ratingCountSinceLastUpdate;
                    average = _ratingCountSinceLastUpdate;
                }

                if (count == 0) return String.Empty;
                if ((average >= 4.4) && (average != 5))
                {
                    return string.Format("{0}/{1:0.00}", count, average);
                }
                else
                {
                    return string.Format("{0}/{1:0.0}", count, average);
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}