﻿using System;
using System.IO.IsolatedStorage;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using Microsoft.Phone.Marketplace;
using Microsoft.Phone.Controls;
using System.Xml.Linq;

namespace AppReviews
{
    public static class Globals
    {
        public static Uri MainPageUri = new Uri("/MainPage.xaml", UriKind.Relative);
        public static Uri AddAppPageUri = new Uri("/AddAppPage.xaml", UriKind.Relative);
        public static Uri RemoveAppPageUri = new Uri("/RemoveAppPage.xaml", UriKind.Relative);

        public static Dispatcher Dispatcher = null;
        public static bool IsGoingToMarketPlace = false;

        public static bool SameSession = false;
        private static class Constants
        {
            internal const string DataFileName = "AppReviews.dat";
        }

        public static PerformanceProgressBar Progress = null;
        private static bool _isLoading = false;
        public static bool IsLoading
        {
            get
            {
                return _isLoading;
            }
            set
            {
                if (_isLoading == value) return;
                _isLoading = value;
                UpdateLoadingView();
            }
        }

        public static TextBlock StatusTextControl = null;
        public static string StatusText
        {
            set
            {
                if ((StatusTextControl == null) || (Dispatcher == null)) return;
                Dispatcher.BeginInvoke(delegate
                {
                    if (StatusTextControl != null)
                    {
                        StatusTextControl.Text = value;
                    }
                });
            }
        }

        public static void UpdateLoadingView()
        {
            if ((Progress == null) || (Dispatcher == null)) return;
            Dispatcher.BeginInvoke(delegate
            {
                if (Progress != null)
                {
                    Progress.IsLoading = _isLoading;
                }
            });
        }
        

        public static string SearchTerm = String.Empty;

        public static ErrorControl ErrorControl = null;
        private static string _errorMessage = null;
        public static string ErrorMessage
        {
            get
            {
                return _errorMessage;
            }
            set
            {
                if (_errorMessage == value) return;
                _errorMessage = value;
                UpdateErrorView();
            }
        }

        public static void UpdateErrorView()
        {
            if (ErrorControl == null) return;

            ErrorControl.ErrorMessage = _errorMessage;
        }

        public static AppsData Data = new AppsData();
        public static void LoadStateIfNeeded()
        {
            if (SameSession) return; // state is already loaded correctly
            SameSession = true;

            // load persistent state
            try
            {
                IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForApplication();
                IsolatedStorageFileStream file = storage.OpenFile(Constants.DataFileName, System.IO.FileMode.Open);
                byte[] data = new byte[file.Length];
                file.Read(data, 0, data.Length);
                file.Close();
                string stringData = System.Text.UTF8Encoding.UTF8.GetString(data, 0, data.Length);

                string[] items = stringData.Split('\n');
                int count = (items.Length / 2) * 2;

                Data.Apps.Clear();
                for (int i = 0; i < count; i += 2)
                {
                    if (string.IsNullOrEmpty(items[i])) items[i] = items[i + 1];
                    Data.Apps.Add(new ApplicationData() { Name = items[i], Id = items[i + 1] });
                }
            }
            catch
            {
#if DEBUG
                Data.Apps.Add(new ApplicationData() { Id = "1e2ccc81-d2f9-df11-9264-00237de2db9e" });
#endif
            }

        }

        public static void SaveState()
        {
            // save persistent state
            try
            {
                byte[] data;

                StringBuilder sb = new StringBuilder();

                foreach (ApplicationData app in Data.Apps)
                {
                    sb.AppendFormat("{0}\n{1}\n", app.Name, app.Id);
                }

                data = System.Text.UTF8Encoding.UTF8.GetBytes(sb.ToString());
                IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForApplication();
                IsolatedStorageFileStream file = storage.OpenFile(Constants.DataFileName, System.IO.FileMode.Create);
                file.Write(data, 0, data.Length);
                file.Close();
            }
            catch
            {
            }
        }


        public static LicenseInformation _license = new LicenseInformation();

        public static bool IsTrial
        {
            get
            {
#if DEBUG
                return MessageBox.Show("CLICK OK TO SIMULATE FULL LICENSE", "BUY NOW!", MessageBoxButton.OKCancel) != MessageBoxResult.OK;
#else
				return _license.IsTrial();
#endif
            }
        }

    }
}
