﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using AppReviews.Web;
using Microsoft.Advertising.Mobile.UI;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Tasks;
using System.Xml.Linq;

namespace AppReviews
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        public MainPage()
        {
            Globals.Dispatcher = Dispatcher;
            Globals.IsGoingToMarketPlace = false; // Reset flag to prevent double-tapping the 'show in marketplace' function
            InitializeComponent();
            //adControl.AdControlError += new EventHandler<ErrorEventArgs>(adControl_AdControlError);

            if (Globals.IsTrial)
            {
                panelAds.Children.Add(new AdControl("91a50422-0222-42a9-a979-6a27b1f47b10", "26398", true)
                {
                    HorizontalAlignment = HorizontalAlignment.Stretch,
                    Height = 80,
                    Width = 480
                });

                panelAds.Visibility = Visibility.Visible;
            }

            listApps.DataContext = Globals.Data;
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            WebApi.CancelAll();
            Globals.Data.LoadData(null, true);

            Globals.ErrorControl = errorControl;
            Globals.UpdateErrorView();

            Globals.Progress = progressLoad;
            Globals.StatusTextControl = textStatus;
            Globals.UpdateLoadingView();

            if (listApps.Items.Count > 0) listApps.SelectedIndex = 0;
        }

        protected override void OnNavigatingFrom(System.Windows.Navigation.NavigatingCancelEventArgs e)
        {
            Globals.ErrorControl = null;
            Globals.Progress = null;
            base.OnNavigatingFrom(e);
        }

        public MarketplaceDetailTask _buyTask;

        private void btnBuy_Click(object sender, RoutedEventArgs e)
        {
            if (_buyTask == null) { _buyTask = new MarketplaceDetailTask(); }
            _buyTask.Show();
        }

        private void panelError_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            Globals.ErrorMessage = null;
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            NavigationService.Navigate(Globals.AddAppPageUri);
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            NavigationService.Navigate(Globals.RemoveAppPageUri);
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            Globals.ErrorMessage = null;
            WebApi.CancelAll();

            if (listApps.SelectedIndex >= 0)
            {
                ApplicationData app = ((ApplicationData)listApps.SelectedItem);
                if (rootPivot.SelectedIndex == 2)// if in reviews tab
                {
                    Globals.Data.LoadData(app, false);
                    app.LoadReviews();
                }
                else
                {
                    Globals.Data.LoadData(app, true);
                }

                app.MoveToTop();
            }
            else
            {
                Globals.Data.LoadData(null, true);
            }
        }

        private void listApps_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (listApps.SelectedIndex < 0)
            {
                listMarkets.DataContext = null;
                listReviews.DataContext = null;
                rootPivot.Title = string.Format("App Rating - Select App");
                return;
            }

            ApplicationData app = (ApplicationData)listApps.SelectedItem;
            app.MoveToTop();
            rootPivot.Title = string.Format("App Rating - {0}", app.Name);
        }

        private void rootPivot_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (listApps.SelectedIndex < 0)
            {
                ((ApplicationBarIconButton)ApplicationBar.Buttons[2]).IsEnabled = false;
                return;
            }

            ((ApplicationBarIconButton)ApplicationBar.Buttons[2]).IsEnabled = true;
            ApplicationData app = (ApplicationData)listApps.SelectedItem;
            if (rootPivot.SelectedIndex == 1)
            {
                listMarkets.DataContext = app;
                return;
            }

            if (rootPivot.SelectedIndex == 2)
            {
                listReviews.DataContext = app;
                app.LoadReviews();
                return;
            }
        }

        MarketplaceDetailTask _marketTask = null;
        private void btnOpen_Click(object sender, EventArgs e)
        {
            if (listApps.SelectedIndex < 0)
            {
                return;
            }

            if (!Globals.IsGoingToMarketPlace) // Fixed 'double tap' bug, causing 'Navigation is not allowed when the task is not in the foreground'
            {
                Globals.IsGoingToMarketPlace = true;
                ApplicationData app = (ApplicationData)listApps.SelectedItem;
                if (_marketTask == null) _marketTask = new MarketplaceDetailTask();
                _marketTask.ContentType = MarketplaceContentType.Applications;
                _marketTask.ContentIdentifier = app.Id;
                _marketTask.Show();
            }
            
        }

        private void TextBlock_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            ReviewData data = (ReviewData) ((TextBlock) sender).DataContext;
            WebApi.QueueTranslate(data.ReviewText, data, null, TranslateCompleted);
        }

        void TranslateCompleted(WebWorkItem item)
        {
            Globals.Dispatcher.BeginInvoke(delegate
            {
                if (item.Error != null)
                {
                    Globals.ErrorMessage = "Connection error during translation. Try translating again later. Error message: " +
                        item.Error.Message;
                    return;
                }

                if (item.Result == null) // cancelled
                {
                    return;
                }

                ReviewData data = (ReviewData)item.UserData;

                // process result
                try
                {
                    XDocument document = XDocument.Parse(item.Result);
                    data.ReviewText = document.Root.Value;
                }
                catch (Exception e)
                {
                    WebApi.CancelAll();
                    Globals.ErrorMessage = "Incorrect translation data. Try translating again later. Error message: " + e.Message;
                }
            });
        }

        private void About_Click(object sender, EventArgs e)
        {
            MessageBox.Show("App Rating v1.9. For technical support and feedback email mimisunlee@gmail.com", "Information", MessageBoxButton.OK);
        }
    }
}


China
Malaysia
Philippines
Argentina
Peru