﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace AppReviews
{
    public partial class RatingControl : Grid
    {
        bool _isInitialized = false;
        public RatingControl()
        {
            InitializeComponent();
            _isInitialized = true;
            this.Loaded += RatingControl_Loaded;
        }

        void RatingControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.Loaded -= RatingControl_Loaded;
            UpdateView();
        }

        static SolidColorBrush _brushOn = new SolidColorBrush(Colors.White);
        static SolidColorBrush _brushOff = new SolidColorBrush(Colors.Gray);

        double _rating = -1;
        public double Rating
        {
            get { return (double)GetValue(RatingProperty); }
            set
            {
                if (_rating == value) return;
                _rating = value;
                if (_rating < 0) _rating = 0;
                if (_rating > 5) _rating = 5;

                if (!_isInitialized) return;
                UpdateView();

                SetValue(RatingProperty, _rating);
            }
        }

        Rect _rect = new Rect(0, 0, 170, 40);
        private void UpdateView()
        {
            double width = 116;// ActualWidth - 4;
            if (width < 0) width = 0;

            if (_rating <= 0)
            {
                _rect.Width = 0;
            }
            else
            {
                _rect.Width = width * _rating / 5;
            }
            rectClip.Rect = _rect;
        }

        public static readonly DependencyProperty RatingProperty = DependencyProperty.Register("Rating", typeof(double), typeof(RatingControl), new PropertyMetadata(0.0, new PropertyChangedCallback(RatingChanged)));

        public static void RatingChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            RatingControl control = (RatingControl)d;
            control._rating = (double)e.NewValue;
            if (control._rating < 0) control._rating = 0;
            if (control._rating > 5) control._rating = 5;

            if (!control._isInitialized) return;
            control.UpdateView();
        }


    }
}
